@@ -29,15 +29,17 @@ setClass("antsLog",
 # ==========================================================================================================
 # Master function to process ACO tasks
 # ==========================================================================================================
-master_ACO <- function(sIn, fIn, sOut, ind.vl, solspace, setup, extargs, time.str, time.lim, trace, pbars, par.clust) {
+master_ACO <- function(sIn, fIn, sOut, ind.vl, solspace, setup, extargs, time.str, time.lim, pbars, par.clust) {
   # set heuristic parameters based on defaults and user specifications
   param <- setParams_ACO(setup, length(fIn))
 
   # set pheromones based on the solution space and initial parameters
+  message("** Initializing decision network...")
   phero <- setEnvir_ACO(solspace, param)
 
   # perform exploration
-  res <- run_ACO(sIn, fIn, sOut, ind.vl, param, phero, solspace$sp.base, extargs, time.str, time.lim, trace, pbars, par.clust)
+  message("** Optimizing structural parameters...")
+  res <- run_ACO(sIn, fIn, sOut, ind.vl, param, phero, solspace$sp.base, extargs, time.str, time.lim, pbars, par.clust)
 
   # correct the howCalled of best model
   res$model@howCalled@string <- getCall_ACO(res$sol.vec, sIn, fIn, res$sol.args, solspace$sp.base, extargs)
@@ -427,7 +429,7 @@ getCall_ACO <- function(ant, sIn, fIn, args, base, extargs) {
     if (length(s.used) == ncol(sIn)) {
       s.str <- "sIn = sIn"
     } else if (length(s.used) == 1) {
-      s.str <- paste("sIn = sIn[,", s.used, ",drop=F]", sep = "")
+      s.str <- paste("sIn = sIn[,", s.used, ",drop=FALSE]", sep = "")
     } else if (all(diff(s.used) == 1)) {
       s.str <- paste("sIn = sIn[,", paste(range(s.used), collapse = ":"), "]", sep = "")
     } else {
@@ -586,7 +588,7 @@ getLog_ACO <- function(sIn, fIn, log.vec, log.fitness, base, extargs) {
 #' @importFrom doFuture registerDoFuture
 #' @importFrom future plan cluster
 #' @importFrom progressr with_progress progressor
-eval_loocv_ACO <- function(sIn, fIn, sOut, extargs, base, ants, time.str, time.lim, trace, pbars, par.clust) {
+eval_loocv_ACO <- function(sIn, fIn, sOut, extargs, base, ants, time.str, time.lim, pbars, par.clust) {
   # recover population size
   n.pop <- nrow(ants)
 
@@ -601,12 +603,8 @@ eval_loocv_ACO <- function(sIn, fIn, sOut, extargs, base, ants, time.str, time.l
       result <- foreach(i = 1:n.pop, .errorhandling = "pass") %dopar% {
         dt <- difftime(Sys.time(), time.str, units = 'secs')
         if (dt < time.lim) {
-          if (trace) {
-            cat("\n")
-            amf <- fitNtests_ACO(ants[i,], sIn, fIn, sOut, extargs, base)
-          } else {
-            amf <- quiet(fitNtests_ACO(ants[i,], sIn, fIn, sOut, extargs, base))
-          }
+          # build and validate the model
+          amf <- fitNtests_ACO(ants[i,], sIn, fIn, sOut, extargs, base)
           if (pbars) p()
           return(amf)
         } else {
@@ -623,12 +621,8 @@ eval_loocv_ACO <- function(sIn, fIn, sOut, extargs, base, ants, time.str, time.l
     # evaluate the ants as models
     result <- list()
     for (i in 1:n.pop) {
-      if (trace) {
-        cat("\n")
-        result[[i]] <- fitNtests_ACO(ants[i,], sIn, fIn, sOut, extargs, base)
-      } else {
-        result[[i]] <- quiet(fitNtests_ACO(ants[i,], sIn, fIn, sOut, extargs, base))
-      }
+      # build and validate the model
+      result[[i]] <- fitNtests_ACO(ants[i,], sIn, fIn, sOut, extargs, base)
       if (pbars) setTxtProgressBar(pb, i)
 
       # check if we are still on time
@@ -657,7 +651,7 @@ eval_loocv_ACO <- function(sIn, fIn, sOut, extargs, base, ants, time.str, time.l
 #' @importFrom doFuture registerDoFuture
 #' @importFrom future plan cluster
 #' @importFrom progressr with_progress progressor
-eval_houtv_ACO <- function(sIn, fIn, sOut, extargs, base, ants, ind.vl, time.str, time.lim, trace, pbars, par.clust) {
+eval_houtv_ACO <- function(sIn, fIn, sOut, extargs, base, ants, ind.vl, time.str, time.lim, pbars, par.clust) {
   # recover population size and number of replicates
   n.pop <- nrow(ants)
   n.rep <- ncol(ind.vl)
@@ -681,14 +675,9 @@ eval_houtv_ACO <- function(sIn, fIn, sOut, extargs, base, ants, ind.vl, time.str
             # identify active inputs of both types
             active <- getActiveIn_ACO(ants[i,], sIn, fIn, base)
 
-            if (trace) {
-              cat("\n")
-              sub_result[[j]] <- fitNtests_ACO(ants[i,], data$sIn.tr, data$fIn.tr, data$sOut.tr, extargs, base,
-                                               ind.vl, data$sIn.vl, data$fIn.vl, data$sOut.vl, active)
-            } else {
-              sub_result[[j]] <- quiet(fitNtests_ACO(ants[i,], data$sIn.tr, data$fIn.tr, data$sOut.tr, extargs, base,
-                                                     ind.vl, data$sIn.vl, data$fIn.vl, data$sOut.vl, active))
-            }
+            # build and validate the model
+            sub_result[[j]] <- fitNtests_ACO(ants[i,], data$sIn.tr, data$fIn.tr, data$sOut.tr, extargs, base,
+                                             ind.vl, data$sIn.vl, data$fIn.vl, data$sOut.vl, active)
 
             # check if we are still on time
             dt <- difftime(Sys.time(), time.str, units = 'secs')
@@ -705,6 +694,7 @@ eval_houtv_ACO <- function(sIn, fIn, sOut, extargs, base, ants, ind.vl, time.str
           argsList <- lapply(sub_result[done], `[[`, 1)
           modelList <- lapply(sub_result[done], `[[`, 2)
           fitnessVec <- sapply(sub_result[done], `[[`, 3)
+          antMat <- t(sapply(sub_result[done], `[[`, 4))
 
           # identify crashes and usable models
           ids.cr <- which(is.na(fitnessVec))
@@ -716,7 +706,8 @@ eval_houtv_ACO <- function(sIn, fIn, sOut, extargs, base, ants, ind.vl, time.str
             piv <- sample.vec(which(fitnessVec == max(fitnessVec)))
             args <- argsList[[piv]]
             model <- modelList[[piv]]
-            amf <- list(args, model, fitness)
+            ant <- antMat[piv,]
+            amf <- list(args, model, fitness, ant)
           } else {
             amf <- NULL
           }
@@ -740,17 +731,13 @@ eval_houtv_ACO <- function(sIn, fIn, sOut, extargs, base, ants, ind.vl, time.str
       for (j in 1:n.rep) {
         # split data into training and validation
         data <- splitData(sIn, fIn, sOut, ind.vl[,j])
+
         # identify active inputs of both types
         active <- getActiveIn_ACO(ants[i,], sIn, fIn, base)
-        # evaluate model
-        if (trace) {
-          cat("\n")
-          sub_result[[j]] <- fitNtests_ACO(ants[i,], data$sIn.tr, data$fIn.tr, data$sOut.tr, extargs, base,
-                                           ind.vl, data$sIn.vl, data$fIn.vl, data$sOut.vl, active)
-        } else {
-          sub_result[[j]] <- quiet(fitNtests_ACO(ants[i,], data$sIn.tr, data$fIn.tr, data$sOut.tr, extargs, base,
-                                                 ind.vl, data$sIn.vl, data$fIn.vl, data$sOut.vl, active))
-        }
+
+        # build and validate the model
+        sub_result[[j]] <- fitNtests_ACO(ants[i,], data$sIn.tr, data$fIn.tr, data$sOut.tr, extargs, base,
+                                         ind.vl, data$sIn.vl, data$fIn.vl, data$sOut.vl, active)
 
         # check if we are still on time
         dt <- difftime(Sys.time(), time.str, units = 'secs')
@@ -761,11 +748,13 @@ eval_houtv_ACO <- function(sIn, fIn, sOut, extargs, base, ants, ind.vl, time.str
           break
         }
       }
+
       # extract complete evaluations
       done <- which(!sapply(sub_result, is.null))
       argsList <- lapply(sub_result[done], `[[`, 1)
       modelList <- lapply(sub_result[done], `[[`, 2)
       fitnessVec <- sapply(sub_result[done], `[[`, 3)
+      antMat <- t(sapply(sub_result[done], `[[`, 4))
 
       # identify crashes and usable models
       ids.cr <- which(is.na(fitnessVec))
@@ -777,7 +766,8 @@ eval_houtv_ACO <- function(sIn, fIn, sOut, extargs, base, ants, ind.vl, time.str
         piv <- sample.vec(which(fitnessVec == max(fitnessVec)))
         args <- argsList[[piv]]
         model <- modelList[[piv]]
-        result[[i]] <- list(args, model, fitness)
+        ant <- antMat[piv,]
+        result[[i]] <- list(args, model, fitness, ant)
       } else {
         result[[i]] <- NULL
       }
@@ -805,13 +795,13 @@ eval_houtv_ACO <- function(sIn, fIn, sOut, extargs, base, ants, ind.vl, time.str
 # ==========================================================================================================
 fitNtests_ACO <- function(ant, sIn, fIn, sOut, extargs, base,
                             ind.vl = NULL, sIn.vl = NULL, fIn.vl = NULL, sOut.vl = NULL, active = NULL) {
-  b.model <- b.args <- NULL
   args <- formatSol_ACO(ant, sIn, fIn, base)
   poterr <- tryCatch(
     {
-      model <- fgpm(sIn = args$sIn, fIn = args$fIn, sOut = sOut, kerType = args$kerType,
-                     f_disType = args$f_disType, f_pdims = args$f_pdims, f_basType = args$f_basType,
-                     nugget = extargs$nugget, n.starts = extargs$n.starts, n.presample = extargs$n.presample)
+      model <- suppressMessages(fgpm(sIn = args$sIn, fIn = args$fIn, sOut = sOut, kerType = args$kerType,
+                                     f_disType = args$f_disType, f_pdims = args$f_pdims, f_basType = args$f_basType,
+                                     nugget = extargs$nugget, n.starts = extargs$n.starts, n.presample = extargs$n.presample,
+                                     trace = F, pbars = F))
     },
     error = function(e) e
   )
@@ -823,7 +813,7 @@ fitNtests_ACO <- function(ant, sIn, fIn, sOut, extargs, base,
     model <- poterr
     fitness <- NA
   }
-  return(list(args[-(1:2)], model, fitness))
+  return(list(args[-(1:2)], model, fitness, ant))
 }
 # ==========================================================================================================
 
@@ -833,10 +823,10 @@ fitNtests_ACO <- function(ant, sIn, fIn, sOut, extargs, base,
 # Function to prepare input data structures for prediction based on a fgpm arguments
 # ==========================================================================================================
 #' @title Preparation of inputs for predictions based on an fgpm modelCall
-#' @description This function prepared input data structures according to the active inputs specified by a
-#'   \code{"\linkS4class{modelCall}"} object. This function is intended to easily adapt the data structures
-#'   to the requirements of a specific model delivered by the model factory function
-#'   \link[funGp]{fgpm_factory}.
+#' @description \strong{Deprecated function, use \link[funGp]{get_active_in} instead.}\cr This function prepared
+#'   input data structures according to the active inputs specified by a \code{"\linkS4class{modelCall}"}
+#'   object. This function is intended to easily adapt the data structures to the requirements of a specific
+#'   model delivered by the model factory function \link[funGp]{fgpm_factory}.
 #'
 #' @param sIn.pr sIn.pr an optional matrix of scalar input coordinates at which the output values should be
 #'   predicted. Each column is interpreted as a scalar input variable and each row as a coordinate.
@@ -861,85 +851,385 @@ fitNtests_ACO <- function(ant, sIn, fIn, sOut, extargs, base,
 #' \emph{RISCOPE project}.
 #' \href{https://hal.archives-ouvertes.fr/hal-02536624}{[HAL]}
 #'
+#' @seealso \strong{*} \link[funGp]{get_active_in} for the substitute of this function in future releases;
 #' @seealso \strong{*} \link[funGp]{predict} for predictions based on a funGp model;
 #' @seealso \strong{*} \link[funGp]{fgpm} for creation of a funGp model;
 #' @seealso \strong{*} \link[funGp]{fgpm_factory} for funGp heuristic model selection.
 #'
 #' @importFrom qdapRegex rm_between
+#' @rdname package-deprecated
 #' @export
 format4pred <- function(sIn.pr = NULL, fIn.pr = NULL, args) {
-  s.on <- f.on <- F
-  sIn.4mat <- fIn.4mat <- NULL
+  .Deprecated("get_active_in")
+  return(get_active_in(sIn.pr, fIn.pr, args))
+}
+# ==========================================================================================================
 
-  key <- args@string # extract the model call
-  if (grepl("sIn", key)) { # is any scalar input active?
-    s.on <- T
+
+
+# ==========================================================================================================
+# Function to obtain the indices of the variables kept active in some structure delivered by the factory
+# ==========================================================================================================
+#' @title Indices of active inputs in a given model structure
+#' @description The \link[funGp]{fgpm_factory} function returns an object of class \code{"\linkS4class{Xfgpm}"}
+#'   with the function call of all the evaluated models stored in the \code{@log.success@args} and
+#'   \code{@log.crashes@args} slots. The \code{which_on} function interprets the arguments linked to any
+#'   structural configuration and returns a list with two elements: (i) an \code{array} of indices of the scalar
+#'   inputs kept active; and (ii) an \code{array} of indices of the functional inputs kept active.
+#'
+#' @param sIn sIn an optional matrix of scalar input coordinates with all the orignal scalar input variables.
+#'   This is used only to know the total number of scalar input variables. Any \code{matrix} with as many
+#'   columns as original scalar input variables could be used instead.
+#' @param fIn an optional list of functional input coordinates with all the original functional input
+#'   variables. This is used only to know the total number of functional input variables. Any \code{list}
+#'   with as many elements as original functional input variables could be used instead.
+#' @param args an object of class \code{"\linkS4class{modelCall}"}, which specifies the model structure for
+#'   which the active inputs should be extracted.
+#'
+#' @return An object of class \code{"list"}, containing the following information extracted from the
+#'   \emph{args} parameter: (i) an array of indices of the scalar inputs kept active; and (ii) an array of
+#'   indices of the functional inputs kept active.
+#'
+#' @author José Betancourt, François Bachoc and Thierry Klein
+#'
+#' @references Betancourt, J., Bachoc, F., and Klein, T. (2020),
+#' R Package Manual: "Gaussian Process Regression for Scalar and Functional Inputs with funGp - The in-depth tour".
+#' \emph{RISCOPE project}.
+#' \href{https://hal.archives-ouvertes.fr/hal-02536624}{[HAL]}
+#'
+#' @seealso \strong{*} \link[funGp]{get_active_in} for details how to obtain the data structures linked to the
+#' active inputs.
+#' @seealso \strong{*} \linkS4class{modelCall} for details on the \emph{args} argument.
+#' @seealso \strong{*} \link[funGp]{fgpm_factory} for funGp heuristic model selection.
+#' @seealso \strong{*} \linkS4class{Xfgpm} for details on object delivered by \link[funGp]{fgpm_factory}.
+#'
+#' @examples
+#' # extracting the indices of the active inputs in an optimized model________________________
+#' # generating input and output data
+#' set.seed(100)
+#' n.tr <- 32
+#' sIn <- expand.grid(x1 = seq(0,1,length = n.tr^(1/5)), x2 = seq(0,1,length = n.tr^(1/5)),
+#'                    x3 = seq(0,1,length = n.tr^(1/5)), x4 = seq(0,1,length = n.tr^(1/5)),
+#'                    x5 = seq(0,1,length = n.tr^(1/5)))
+#' fIn <- list(f1 = matrix(runif(n.tr*10), ncol = 10), f2 = matrix(runif(n.tr*22), ncol = 22))
+#' sOut <- fgp_BB7(sIn, fIn, n.tr)
+#' \donttest{
+#' # optimizing the model structure with fgpm_factory (~12 seconds)
+#' xm <- fgpm_factory(sIn = sIn, fIn = fIn, sOut = sOut)
+#'
+#' # active inputs in the best model
+#' xm@log.success@args[[1]] # the full fgpm call
+#' which_on(sIn, fIn, xm@log.success@args[[1]]) # only the indices extracted bu which_on
+#' }
+#'
+#' @importFrom qdapRegex rm_between
+#' @export
+which_on <- function (sIn = NULL, fIn = NULL, args) {
+  # initalize controllers and outputs of the function
+  s.on <- f.on <- FALSE
+  s.inds <- f.inds <- NULL
+
+  # determine if there are scalars and functional inputs active
+  key <- args@string
+  if (grepl("sIn", key)) {
+    s.on <- TRUE
   }
-  if (grepl("fIn", key)) { # is any functional input active?
-    f.on <- T
+  if (grepl("fIn", key)) {
+    f.on <- TRUE
   }
 
-  if (all(s.on, f.on)) { # there both, scalar and funtional inputs active
-    # formatting scalar inputs ----------------------------------------------------
-    x <- rm_between(key, "sIn = ", ", fIn", extract = TRUE)[[1]] # extract the fragment of the call related to the scalar inputs
-    if (grepl("\\d", x)) { # does the fragment include input indices?
-      if (grepl("\\(", x)) { # are they grouped in a vector?
-        sIn.4mat <- sIn.pr[, as.numeric(unlist(regmatches(x, gregexpr("[[:digit:]]+", x)))), drop=F]
-      } else if (grepl("drop", x)) {
-        sIn.4mat <- sIn.pr[, as.numeric(unlist(regmatches(x, gregexpr("[[:digit:]]+", x)))), drop=F]
-      } else {
-        v <- as.numeric(unlist(regmatches(x, gregexpr("[[:digit:]]+", x))))
-        sIn.4mat <- sIn.pr[, v[1]:v[2]]
+  # get sIn and fIn expressions when necessary
+  if (all(s.on, f.on)) { # scalar and functional inputs on
+    xs <- rm_between(key, "sIn = ", ", fIn", extract = TRUE)[[1]]
+    xf <- rm_between(key, "fIn = ", ", sOut", extract = TRUE)[[1]]
+  } else if (s.on) { # only scalar inputs on
+    xs <- rm_between(key, "sIn = ", ", sOut", extract = TRUE)[[1]]
+    xf <- fIn.on <- NULL
+  } else if (f.on) { # only functional inputs on
+    xs <- sIn.on <- NULL
+    xf <- rm_between(key, "fIn = ", ", sOut", extract = TRUE)[[1]]
+  } else return(NULL)
+
+  # prune the scalar inputs based on vector of indices
+  if (!is.null(xs)) {
+    # not all the variables are kept on
+    if (grepl("\\d", xs)) {
+      # separated indices of variables in the form c(1,3,4)
+      if (grepl("\\(", xs)) {
+        s.inds <- as.numeric(unlist(regmatches(xs, gregexpr("[[:digit:]]+", xs))))
+      }
+      # consecutive indices of variables in the form 2:5
+      else if (grepl(":", xs)) {
+        v <- as.numeric(unlist(regmatches(xs, gregexpr("[[:digit:]]+", xs))))
+        s.inds <- v[1]:v[2]
       }
-
-    } else {
-      sIn.4mat <- sIn.pr
+      # only one index in the form 4
+      else {
+        s.inds <- as.numeric(unlist(regmatches(xs, gregexpr("[[:digit:]]+", xs))))
+      }
+    }
+    # all the scalar variables are kept on
+    else {
+      s.inds <- 1:ncol(sIn)
     }
+  }
 
-    # formatting functional inputs ----------------------------------------------------
-    x <- rm_between(key, "fIn = ", ", sOut", extract = TRUE)[[1]] # extract the fragment of the call related to the functional inputs
-    if (grepl("\\d", x)) { # does the fragment include input indices?
-      if (grepl("\\(", x)) { # are they grouped in a vector?
-        fIn.4mat <- fIn.pr[[as.numeric(unlist(regmatches(x, gregexpr("[[:digit:]]+", x))))]]
-      } else {
-        fIn.4mat <- fIn.pr[as.numeric(rm_between(x, "[", "]", extract = TRUE)[[1]])]
+  # prune the functional inputs based on vector of indices
+  if (!is.null(xf)) {
+    # not all the variables are kept on
+    if (grepl("\\d", xf)) {
+      # separated indices of variables in the form c(1,3,4)
+      if (grepl("\\(", xf)) {
+        f.inds <- as.numeric(unlist(regmatches(xf, gregexpr("[[:digit:]]+", xf))))
+      }
+      # consecutive indices of variables in the form 2:5
+      else if (grepl(":", xf)) {
+        v <- as.numeric(unlist(regmatches(xf, gregexpr("[[:digit:]]+", xf))))
+        f.inds <- v[1]:v[2]
+      }
+      # only one index in the form 4
+      else {
+        f.inds <- as.numeric(unlist(regmatches(xf, gregexpr("[[:digit:]]+", xf))))
       }
-    } else {
-      fIn.4mat <- fIn.pr
     }
+    # all the variables are kept on
+    else {
+      f.inds <- 1:length(fIn)
+    }
+  }
+
+  return(list(s.inds = s.inds, f.inds = f.inds))
+}
+# ==========================================================================================================
 
-  } else if (s.on) { # there are only scalar inputs active
-    # formatting scalar inputs ----------------------------------------------------
-    x <- rm_between(key, "sIn = ", ", sOut", extract = TRUE)[[1]] # extract the fragment of the call related to the scalar inputs
-    if (grepl("\\d", x)) { # does the fragment include input indices?
-      if (grepl("\\(", x)) { # are they grouped in a vector?
-        sIn.4mat <- sIn.pr[, as.numeric(unlist(regmatches(x, gregexpr("[[:digit:]]+", x)))), drop=F]
-      } else if (grepl("drop", x)) {
-        sIn.4mat <- sIn.pr[, as.numeric(unlist(regmatches(x, gregexpr("[[:digit:]]+", x)))), drop=F]
-      } else {
-        v <- as.numeric(unlist(regmatches(x, gregexpr("[[:digit:]]+", x))))
-        sIn.4mat <- sIn.pr[, v[1]:v[2]]
-      }
+
 
-    } else {
-      sIn.4mat <- sIn.pr
-    }
+# ==========================================================================================================
+# Function to prune the inputs data structures according to some structural parameters delivered by the factory
+# ==========================================================================================================
+#' @title Extraction of active inputs in a given model structure
+#' @description The \link[funGp]{fgpm_factory} function returns an object of class \code{"\linkS4class{Xfgpm}"}
+#'   with the function call of all the evaluated models stored in the \code{@log.success@args} and
+#'   \code{@log.crashes@args} slots. The \code{get_active_in} function interprets the arguments linked to any
+#'   structural configuration and returns a list with two elements: (i) a \code{matrix} of scalar input
+#'   variables kept active; and (ii) a \code{list} of functional input variables kept active.
+#'
+#' @param sIn sIn an optional matrix of scalar input coordinates with all the orignal scalar input variables.
+#' @param fIn an optional list of functional input coordinates with all the original functional input
+#'   variables.
+#' @param args an object of class \code{"\linkS4class{modelCall}"}, which specifies the model structure for
+#'   which the active inputs should be extracted.
+#'
+#' @return An object of class \code{"list"}, containing the following information extracted from the
+#'   \emph{args} parameter: (i) a \code{matrix} of scalar input variables kept active; and (ii) a \code{list}
+#'   of functional input variables kept active.
+#'
+#' @author José Betancourt, François Bachoc and Thierry Klein
+#'
+#' @references Betancourt, J., Bachoc, F., and Klein, T. (2020),
+#' R Package Manual: "Gaussian Process Regression for Scalar and Functional Inputs with funGp - The in-depth tour".
+#' \emph{RISCOPE project}.
+#' \href{https://hal.archives-ouvertes.fr/hal-02536624}{[HAL]}
+#'
+#' @seealso \strong{*} \link[funGp]{which_on} for details how to obtain only on the indices of the active inputs.
+#' @seealso \strong{*} \linkS4class{modelCall} for details on the \emph{args} argument.
+#' @seealso \strong{*} \link[funGp]{fgpm_factory} for funGp heuristic model selection.
+#' @seealso \strong{*} \linkS4class{Xfgpm} for details on object delivered by \link[funGp]{fgpm_factory}.
+#'
+#' @examples
+#' # extracting the indices of the active inputs in an optimized model________________________
+#' # generating input and output data
+#' set.seed(100)
+#' n.tr <- 32
+#' sIn <- expand.grid(x1 = seq(0,1,length = n.tr^(1/5)), x2 = seq(0,1,length = n.tr^(1/5)),
+#'                    x3 = seq(0,1,length = n.tr^(1/5)), x4 = seq(0,1,length = n.tr^(1/5)),
+#'                    x5 = seq(0,1,length = n.tr^(1/5)))
+#' fIn <- list(f1 = matrix(runif(n.tr*10), ncol = 10), f2 = matrix(runif(n.tr*22), ncol = 22))
+#' sOut <- fgp_BB7(sIn, fIn, n.tr)
+#' \donttest{
+#' # optimizing the model structure with fgpm_factory (~12 seconds)
+#' xm <- fgpm_factory(sIn = sIn, fIn = fIn, sOut = sOut)
+#'
+#' # indices of active inputs in the best model
+#' xm@log.success@args[[1]] # the full fgpm call
+#' which_on(sIn, fIn, xm@log.success@args[[1]]) # only the indices extracted bu which_on
+#'
+#' # data structures of active inputs
+#' active <- get_active_in(sIn, fIn, xm@log.success@args[[1]])
+#' active$sIn.on # scalar data structures
+#' active$fIn.on # functional data structures
+#' }
+#' \donttest{
+#' # preparing new data for prediction based on inputs kept active____________________________
+#' # generating input and output data for structural optimization
+#' set.seed(100)
+#' n.tr <- 32
+#' sIn <- expand.grid(x1 = seq(0,1,length = n.tr^(1/5)), x2 = seq(0,1,length = n.tr^(1/5)),
+#'                    x3 = seq(0,1,length = n.tr^(1/5)), x4 = seq(0,1,length = n.tr^(1/5)),
+#'                    x5 = seq(0,1,length = n.tr^(1/5)))
+#' fIn <- list(f1 = matrix(runif(n.tr*10), ncol = 10), f2 = matrix(runif(n.tr*22), ncol = 22))
+#' sOut <- fgp_BB7(sIn, fIn, n.tr)
+#'
+#' # optimizing the model structure with fgpm_factory (~12 seconds)
+#' xm <- fgpm_factory(sIn = sIn, fIn = fIn, sOut = sOut)
+#'
+#' # identifying selected model and corresponding fgpm arguments
+#' opt.model <- xm@model
+#' opt.args <- xm@log.success@args[[1]]
+#'
+#' # generating new input data for prediction
+#' n.pr <- 243
+#' sIn.pr <- expand.grid(x1 = seq(0,1,length = n.pr^(1/5)), x2 = seq(0,1,length = n.pr^(1/5)),
+#'                       x3 = seq(0,1,length = n.pr^(1/5)), x4 = seq(0,1,length = n.pr^(1/5)),
+#'                       x5 = seq(0,1,length = n.pr^(1/5)))
+#' fIn.pr <- list(f1 = matrix(runif(n.pr*10), ncol = 10), f2 = matrix(runif(n.pr*22), ncol = 22))
+#'
+#' # prunning data structures for prediction to keep only active inputs!!
+#' active <- get_active_in(sIn.pr, fIn.pr, opt.args)
+#'
+#' # making predictions
+#' preds <- predict(opt.model, sIn.pr = active$sIn.on, fIn.pr = active$fIn.on)
+#'
+#' # plotting predictions
+#' plotPreds(opt.model, preds)
+#' }
+#' \donttest{
+#' # preparing new data for simulation based on inputs kept active____________________________
+#' # generating input and output data for structural optimization
+#' set.seed(100)
+#' n.tr <- 32
+#' sIn <- expand.grid(x1 = seq(0,1,length = n.tr^(1/5)), x2 = seq(0,1,length = n.tr^(1/5)),
+#'                    x3 = seq(0,1,length = n.tr^(1/5)), x4 = seq(0,1,length = n.tr^(1/5)),
+#'                    x5 = seq(0,1,length = n.tr^(1/5)))
+#' fIn <- list(f1 = matrix(runif(n.tr*10), ncol = 10), f2 = matrix(runif(n.tr*22), ncol = 22))
+#' sOut <- fgp_BB7(sIn, fIn, n.tr)
+#'
+#' # optimizing the model structure with fgpm_factory (~12 seconds)
+#' xm <- fgpm_factory(sIn = sIn, fIn = fIn, sOut = sOut)
+#'
+#' # identifying selected model and corresponding fgpm arguments
+#' opt.model <- xm@model
+#' opt.args <- xm@log.success@args[[1]]
+#'
+#' # generating new input data for simulation
+#' n.sm <- 243
+#' sIn.sm <- expand.grid(x1 = seq(0,1,length = n.sm^(1/5)), x2 = seq(0,1,length = n.sm^(1/5)),
+#'                       x3 = seq(0,1,length = n.sm^(1/5)), x4 = seq(0,1,length = n.sm^(1/5)),
+#'                       x5 = seq(0,1,length = n.sm^(1/5)))
+#' fIn.sm <- list(f1 = matrix(runif(n.sm*10), ncol = 10), f2 = matrix(runif(n.sm*22), ncol = 22))
+#'
+#' # prunning data structures for simulation to keep only active inputs!!
+#' active <- get_active_in(sIn.sm, fIn.sm, opt.args)
+#'
+#' # making light simulations
+#' sims_l <- simulate(opt.model, nsim = 10, sIn.sm = sIn.sm, fIn.sm = fIn.sm)
+#'
+#' # plotting light simulations
+#' plotSims(opt.model, sims_l)
+#' }
+#' \donttest{
+#' # rebuilding of 3 best models using new data_______________________________________________
+#' # NOTE: this example is of higher complexity than the previous ones. We recomend you run
+#' #       the previous examples and understand the @log.success and @log.crashes slots in
+#' #       the Xfgpm object delivered by fgpm_factory.
+#' #
+#' #       In the second example above we showed how to use get_active_in to prune the input
+#' #       data structures for prediction based on the fgpm arguments of the best model found
+#' #       by fgpm_factory. In this new example we generalize that concept by: (i) rebuilding
+#' #       the 3 best models fouond by fgpm_factory using new data, (ii) pruning the input
+#' #       data structures used for prediction with each of the models, and (iii) plotting
+#' #       the predictions made by the three models. The key ingredient here is that the
+#' #       three best models might have different scalar and functional inputs active. The
+#' #       get_active_in function will allow to process the data structures in order to
+#' #       extract only the scalar inputs required to re-build the model and then to make
+#' #       predictions with each model. Check also the funGp manual for further details
+#' #
+#' #       funGp manual: https://hal.archives-ouvertes.fr/hal-02536624
+#'
+#'
+#' # <<<<<<< PART 1: calling fgpm_factory to perform the structural optimization >>>>>>>
+#' #         -------------------------------------------------------------------
+#' # generating input and output data for structural optimization
+#' set.seed(100)
+#' n.tr <- 32
+#' sIn <- expand.grid(x1 = seq(0,1,length = n.tr^(1/5)), x2 = seq(0,1,length = n.tr^(1/5)),
+#'                    x3 = seq(0,1,length = n.tr^(1/5)), x4 = seq(0,1,length = n.tr^(1/5)),
+#'                    x5 = seq(0,1,length = n.tr^(1/5)))
+#' fIn <- list(f1 = matrix(runif(n.tr*10), ncol = 10), f2 = matrix(runif(n.tr*22), ncol = 22))
+#' sOut <- fgp_BB7(sIn, fIn, n.tr)
+#'
+#' # optimizing the model structure with fgpm_factory (~12 seconds)
+#' xm <- fgpm_factory(sIn = sIn, fIn = fIn, sOut = sOut)
+#'
+#'
+#' # <<<<<<< PART 2: re-building the three best models found by fgpm_factory >>>>>>>
+#' #         ---------------------------------------------------------------
+#' # recovering the fgpm arguments of the three best models
+#' argStack <- xm@log.success@args[1:3]
+#'
+#' # new data arrived, now we have 243 observations
+#' n.nw <- 243 # more points!
+#' sIn.nw <- expand.grid(x1 = seq(0,1,length = n.nw^(1/5)), x2 = seq(0,1,length = n.nw^(1/5)),
+#'                       x3 = seq(0,1,length = n.nw^(1/5)), x4 = seq(0,1,length = n.nw^(1/5)),
+#'                       x5 = seq(0,1,length = n.nw^(1/5)))
+#' fIn.nw <- list(f1 = matrix(runif(n.nw*10), ncol = 10), f2 = matrix(runif(n.nw*22), ncol = 22))
+#' sOut.nw <- fgp_BB7(sIn.nw, fIn.nw, n.nw)
+#'
+#' # re-building the three best models based on the new data (compact code with all 3 calls)
+#' modStack <- lapply(1:3, function(i) eval(parse(text = argStack[[i]]@string)[[1]]))
+#'
+#'
+#' # <<<<<<< PART 3: making predictions from the three best models found by fgpm_factory >>>>>>>
+#' #         ---------------------------------------------------------------------------
+#' # generating input data for prediction
+#' n.pr <- 32
+#' sIn.pr <- expand.grid(x1 = seq(0,1,length = n.pr^(1/5)), x2 = seq(0,1,length = n.pr^(1/5)),
+#'                       x3 = seq(0,1,length = n.pr^(1/5)), x4 = seq(0,1,length = n.pr^(1/5)),
+#'                       x5 = seq(0,1,length = n.pr^(1/5)))
+#' fIn.pr <- list(f1 = matrix(runif(n.pr*10), ncol = 10), matrix(runif(n.pr*22), ncol = 22))
+#'
+#' # making predictions based on the three best models (compact code with all 3 calls)
+#' preds <- do.call(cbind, Map(function(model, args) {
+#'   active <- get_active_in(sIn.pr, fIn.pr, args)
+#'   predict(model, sIn.pr = active$sIn.on, fIn.pr = active$fIn.on)$mean
+#' }, modStack, argStack))
+#'
+#'
+#' # <<<<<<< PART 4: plotting predictions from the three best models found by fgpm_factory >>>>>>>
+#' #         -----------------------------------------------------------------------------
+#' # plotting predictions made by the three models
+#' require(plyr) # for conciseness in plotting the three models
+#' plot(1, xlim = c(1,nrow(preds)), ylim = range(preds), xaxt = "n",
+#'      xlab = "Prediction point index", ylab = "Output",
+#'      main = "Predictions with best 3 structural configurations")
+#' axis(1, 1:nrow(preds))
+#' l_ply(seq_len(n.pr), function(i) lines(rep(i,2), range(preds[i,1:3]), col = "grey35", lty = 3))
+#' points(preds[,1], pch = 21, bg = "black")
+#' points(preds[,2], pch = 23, bg = "red")
+#' points(preds[,3], pch = 24, bg = "green")
+#' legend("bottomleft", legend = c("Model 1", "Model 2", "Model 3"),
+#'        pch = c(21, 23, 24), pt.bg = c("black", "red", "green"), inset = c(.02,.08))
+#' }
+#'
+#' @importFrom qdapRegex rm_between
+#' @export
+get_active_in <- function (sIn = NULL, fIn = NULL, args) {
+  # get indices of active inputs
+  active <- which_on(sIn, fIn, args)
 
-  } else if (f.on) { # there are only functional inputs active
-    # formatting functional inputs ----------------------------------------------------
-    x <- rm_between(key, "fIn = ", ", sOut", extract = TRUE)[[1]] # extract the fragment of the call related to the functional inputs
-    if (grepl("\\d", x)) { # does the fragment include input indices?
-      if (grepl("\\(", x)) { # are they grouped in a vector?
-        fIn.4mat <- fIn.pr[[as.numeric(unlist(regmatches(x, gregexpr("[[:digit:]]+", x))))]]
-      } else {
-        fIn.4mat <- fIn.pr[as.numeric(rm_between(x, "[", "]", extract = TRUE)[[1]])]
-      }
-    } else {
-      fIn.4mat <- fIn.pr
-    }
+  # original inputs: hybrid
+  if (all(!is.null(sIn), !is.null(fIn))) {
+    return(list(sIn.on = sIn[,active$s.inds,drop=FALSE], fIn.on = fIn[active$f.inds]))
+  }
 
-  } else return(NULL)
+  # original inputs: functional
+  else if (!is.null(fIn)) {
+    return(list(sIn.on = NULL, fIn.on = fIn[active$f.inds]))
+  }
 
-  return(list(sIn.pr = sIn.4mat, fIn.pr = fIn.4mat))
+  # original inputs: scalar
+  else if (!is.null(fIn)) {
+    return(list(sIn.on = sIn[,active$s.inds,drop=FALSE], fIn.on = NULL))
+  }
 }
 # ==========================================================================================================
